/*
    Bantumi
    Copyright 2005 - 2009 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#import "BantumiView.h"
#include <sys/time.h>
#include "bantumigl.h"
#include "glfont.h"

#define GAME_NUM_KEY @"gameNum"
#define GAME_LEVEL_KEY @"gameLevel"


long curTime() {
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return tv.tv_sec*1000 + tv.tv_usec/1000;
}

@implementation BantumiView

- (id)initWithCoder:(NSCoder*)coder {
	frontEnd = NULL;
	game = NULL;
	textFont = numFont = NULL;

	if ((self = [super initWithCoder:coder])) {
		frontEnd = new BantumiGL(backingWidth, backingHeight);
		NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
		if ([defaults objectForKey: GAME_NUM_KEY])
			frontEnd->setNum([defaults integerForKey: GAME_NUM_KEY]);
		if ([defaults objectForKey: GAME_LEVEL_KEY])
			frontEnd->setLevel([defaults integerForKey: GAME_LEVEL_KEY]);
		
		char fontPath[250];
//		NSString* path = [[NSBundle mainBundle] pathForResource: @"Vera" ofType: @"ttf"];
//		[path getCString: fontPath maxLength: sizeof(fontPath) encoding: NSUTF8StringEncoding];
//		CFURLRef url = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
		CFURLRef url = CFBundleCopyResourceURL(CFBundleGetMainBundle(), CFSTR("Vera"), CFSTR("ttf"), NULL);
		CFURLGetFileSystemRepresentation(url, true, (UInt8*) fontPath, sizeof(fontPath));
		CFRelease(url);

		numFont = makeGLFont(fontPath, 26, 64, 64, '0', '9');
		textFont = makeGLFont(fontPath, 30, 256, 256, ' ', '\366', 'z'+1, '\366'-1);
		frontEnd->setTextFont(textFont);
		frontEnd->setNumFont(numFont);
		game = new Bantumi(frontEnd);
		
		startTime = curTime();
		inUpdate = NO;
		[self drawView];
	}
	return self;
}

- (void)drawContent {
	if (inUpdate)
		return;
	if (!frontEnd) {
		glClearColor(0, 0, 0, 0);
		glClear(GL_COLOR_BUFFER_BIT);
		return;
	}

	// this may launch modal dialogs, keep outside of the drawing?
	inUpdate = YES;
	game->update();
	inUpdate = NO;
	int time = curTime() - startTime;
	frontEnd->draw(time);

	if (game->exit()) {
		exit(0);
	}
}

- (void) saveState {
	if (frontEnd) {
		NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
		[defaults setInteger: frontEnd->getNum() forKey: GAME_NUM_KEY];
		[defaults setInteger: frontEnd->getLevel() forKey: GAME_LEVEL_KEY];
	}
}

- (void)dealloc {
	[self saveState];
	delete game;
	delete frontEnd;
	delete textFont;
	delete numFont;
	[super dealloc];
}

- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
	if ([[event allTouches] count] > 1) {
		game->pressed(ESCAPE);
	} else {
		UITouch* touch = [touches anyObject];
		CGPoint pos = [touch locationInView: self];
		game->click(pos.x, pos.y, 20);
	}
}

- (void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event {
}

- (void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event {
}

- (void)touchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event {
}

@end
