/*
    Bantumi
    Copyright 2005 - 2008 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#include <windows.h>
#include <tchar.h>
#include <aygshell.h>
#include <GLES/egl.h>
#include "bantumi.h"
#include "bantumigl.h"
#include "glfont.h"
#include "socketconn.h"
#include "gui.h"
#include "win32gui.h"
#include "resource.h"

#define ROOT_KEY HKEY_CURRENT_USER
#define KEY_NAME _T("Software\\martin.st\\BantumiGL")
#define HOSTNAME_NAME _T("TcpHostname")
#define PORT_NAME _T("TcpPort")
#define LEVEL_NAME _T("Level")
#define NUMBER_NAME _T("Number")
#define BLUETOOTH_CACHE_NAME _T("BluetoothCache")
#define BT_DEVICE_NAME _T("Name")
#define BT_DEVICE_ADDR _T("Addr")

EGLDisplay display;
EGLSurface surface;
EGLContext context;
EGLConfig config;
int curWidth, curHeight;

BantumiGL* frontEnd;
GLFont* textFont;
GLFont* numFont;
Bantumi* game;

int lastTime;
BOOL active = TRUE;

HINSTANCE hInst;
HWND window;
bool ppc = false;

TCHAR hostnameBuffer[200];
TCHAR portNumberBuffer[200];

bool lastIngame;

void updateMenu(HWND wnd, bool first = false) {
	if (ppc)
		return;

	bool ingame = false;
	if (game && game->isStarted())
		ingame = true;

	if (!first && ingame == lastIngame)
		return;

	lastIngame = ingame;

	SHMENUBARINFO shmbi;
	ZeroMemory(&shmbi, sizeof(shmbi));
	shmbi.cbSize = sizeof(shmbi);
	shmbi.hwndParent = wnd;
	shmbi.nToolBarId = ingame ? GAME_MENU : MAIN_MENU;
	shmbi.hInstRes = hInst;
	shmbi.dwFlags = SHCMBF_HMENU | SHCMBF_HIDESIPBUTTON;
	SHCreateMenuBar(&shmbi);
}

LRESULT CALLBACK windowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {

	switch (uMsg) {
	case WM_CREATE:
		updateMenu(hWnd, true);
		break;

	case WM_PAINT:
/*
		if (frontEnd) {
			frontEnd->draw(lastTime);
			eglSwapBuffers(display, surface);
		}
*/
		break;

	case WM_CLOSE:
	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	case WM_LBUTTONDOWN:
		game->click(LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_ACTIVATE:
		active = (BOOL) (LOWORD(wParam));
		if (active)
			SHFullScreen(hWnd, SHFS_HIDESIPBUTTON);
		break;
	case WM_SIZE:
		{
			RECT rect;
			GetClientRect(hWnd, &rect);
			int newWidth = rect.right - rect.left; 
			int newHeight = rect.bottom - rect.top;
			if (display != EGL_NO_DISPLAY && (newWidth != curWidth || newHeight != curHeight)) {
				curWidth = newWidth;
				curHeight = newHeight;
				eglDestroySurface(display, surface);

				surface = eglCreateWindowSurface(display, config, hWnd, NULL);
				if (!surface) {
					return 1;
				}
				if (eglMakeCurrent(display, surface, surface, context) == EGL_FALSE) {
					return 1;
				}

				frontEnd->reinitGL(curWidth, curHeight);
			}
			break;
		}

	case WM_KEYDOWN:
		switch (wParam) {
		case VK_ESCAPE:
			game->pressed(ESCAPE);
			break;
		case VK_SPACE:
		case VK_RETURN:
			game->pressed(SELECT);
			break;
		case VK_LEFT:
			game->pressed(LEFT);
			break;
		case VK_RIGHT:
			game->pressed(RIGHT);
			break;
		case VK_UP:
			game->pressed(UP);
			break;
		case VK_DOWN:
			game->pressed(DOWN);
			break;
		}
		break;
	case WM_KEYUP:
		switch (wParam) {
		case VK_F1:
			game->pressed(START);
			break;
		case VK_F2:
			game->pressed(ESCAPE);
			break;
		}
		break;

	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow) {
	hInst = hInstance;

#define WINDOW_TITLE L"Bantumi GL"
#define CLASS_NAME L"BantumiGLClass"

	HWND wnd = FindWindow(CLASS_NAME, WINDOW_TITLE);
	if (wnd) {
		SetForegroundWindow((HWND)((UINT_PTR)wnd | 0x01));
		return 0;
	}


	TCHAR platform[20];
	if (SystemParametersInfo(SPI_GETPLATFORMTYPE, sizeof(platform), platform, 0) != 0) {
		if (!_tcscmp(platform, _T("PocketPC")))
			ppc = true;
	}

	WNDCLASS wc;

	ZeroMemory(&wc, sizeof(wc));
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = windowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
//	wc.hbrBackground = CreateSolidBrush(RGB(0, 0, 0));
	wc.hbrBackground = (HBRUSH) COLOR_WINDOW;
	wc.lpszMenuName = NULL;
	wc.lpszClassName = CLASS_NAME;
	RegisterClass(&wc);

	wnd = CreateWindow(CLASS_NAME, WINDOW_TITLE, WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	window = wnd;


	display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
	eglInitialize(display, NULL, NULL);

	EGLint attribList[] = {
		EGL_BUFFER_SIZE, 16, EGL_DEPTH_SIZE, 16,
		EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
		EGL_NONE
	};

	EGLint numConfig;
	if (eglChooseConfig(display, attribList, &config, 1, &numConfig) == EGL_FALSE) {
		return 1;
	}
	if (numConfig < 1) {
		return 1;
	}
	surface = eglCreateWindowSurface(display, config, wnd, NULL);
	if (!surface) {
		return 1;
	}
#ifdef EGL_VERSION_1_2
	eglBindAPI(EGL_OPENGL_ES_API);
#endif
	context = eglCreateContext(display, config, NULL, NULL);
	if (!context) {
		return 1;
	}

	if (eglMakeCurrent(display, surface, surface, context) == EGL_FALSE) {
		return 1;
	}

	RECT rect;
	GetClientRect(wnd, &rect);
	curWidth = rect.right - rect.left; 
	curHeight = rect.bottom - rect.top;

	initSockets();
	initGui();


	frontEnd = new BantumiGL(curWidth, curHeight);
#define FONT_NAME "Vera.ttf"
	char fontname[500];
	FILE* test = fopen(FONT_NAME, "rb");
	if (test) {
		fclose(test);
		strcpy(fontname, FONT_NAME);
	} else {
		TCHAR path[500];
		GetModuleFileName(NULL, path, sizeof(path)/sizeof(path[0]));
		TCHAR* separator = _tcsrchr(path, '\\');
		if (separator)
			*separator = '\0';
		TCHAR* in = path;
		char* out = fontname;
		while (*in)
			*out++ = (char) *in++;
		*out++ = '\\';
		*out = '\0';
		strcat(out, FONT_NAME);
	}
	numFont = makeGLFont(fontname, 20, 64, 64, '0', '9');
	textFont = makeGLFont(fontname, 25, 256, 256, ' ', '\366', 'z'+1, '\366'-1);
	if (!numFont)
        numFont = new TexGLFont(' ', 'z' - ' ' + 1);
	if (!textFont)
        textFont = new TexGLFont(' ', 'z' - ' ' + 1);

	frontEnd->setTextFont(textFont);
	frontEnd->setNumFont(numFont);
	game = new Bantumi(frontEnd);

	HKEY key;
	if (RegOpenKeyEx(ROOT_KEY, KEY_NAME, 0, 0, &key) == ERROR_SUCCESS) {
		DWORD length = sizeof(hostnameBuffer);
		DWORD type;
		if (RegQueryValueEx(key, HOSTNAME_NAME, NULL, &type, (BYTE*) hostnameBuffer, &length) == ERROR_SUCCESS && type == REG_SZ)
			lastHost = hostnameBuffer;
		length = sizeof(portNumberBuffer);
		if (RegQueryValueEx(key, PORT_NAME, NULL, &type, (BYTE*) portNumberBuffer, &length) == ERROR_SUCCESS && type == REG_SZ)
			lastPort = portNumberBuffer;
		DWORD val;
		length = sizeof(val);
		if (RegQueryValueEx(key, LEVEL_NAME, NULL, &type, (BYTE*) &val, &length) == ERROR_SUCCESS && type == REG_DWORD)
			frontEnd->setLevel(val);
		length = sizeof(val);
		if (RegQueryValueEx(key, NUMBER_NAME, NULL, &type, (BYTE*) &val, &length) == ERROR_SUCCESS && type == REG_DWORD)
			frontEnd->setNum(val);

		HKEY btCache;
		if (RegOpenKeyEx(key, BLUETOOTH_CACHE_NAME, 0, 0, &btCache) == ERROR_SUCCESS) {
			for (int i = 0; ; i++) {
				TCHAR keyName[40];
				DWORD bufLength = sizeof(keyName)/sizeof(keyName[0]);
				LONG retval = RegEnumKeyEx(btCache, i, keyName, &bufLength, NULL, NULL, NULL, NULL);
				if (retval == ERROR_SUCCESS) {
					HKEY device;
					if (RegOpenKeyEx(btCache, keyName, 0, 0, &device) == ERROR_SUCCESS) {
						BYTE addrBuf[sizeof(BT_ADDR)];
						length = sizeof(addrBuf);
						if (RegQueryValueEx(device, BT_DEVICE_ADDR, NULL, &type, addrBuf, &length) != ERROR_SUCCESS || type != REG_BINARY || length != sizeof(addrBuf)) {
							RegCloseKey(device);
							continue;
						}
						BT_ADDR addr = 0;
						for (unsigned int j = 0; j < sizeof(BT_ADDR); j++)
							addr |= ((BT_ADDR) addrBuf[j]) << (8*(sizeof(BT_ADDR) - 1 - j));
						TCHAR nameBuf[100];
						length = sizeof(nameBuf);
						if (RegQueryValueEx(device, BT_DEVICE_NAME, NULL, &type, (BYTE*) nameBuf, &length) != ERROR_SUCCESS || type != REG_SZ) {
							RegCloseKey(device);
							continue;
						}
						addBluetoothDevice(nameBuf, addr);
						RegCloseKey(device);
					}
				} else if (retval == ERROR_NO_MORE_ITEMS) {
					break;
				} else {
					char errBuf[100];
					sprintf(errBuf, "RegEnumKeysEx: %d", (int) retval);
					showWarning(errBuf);
					break;
				}
			}
			RegCloseKey(btCache);
		}

		RegCloseKey(key);
	}



//	DWORD start = GetTickCount();
	LARGE_INTEGER startCounter;
	LARGE_INTEGER counterFreq;
	QueryPerformanceFrequency(&counterFreq);
	QueryPerformanceCounter(&startCounter);
	while (true) {
		MSG msg;
		BOOL gotMsg;

		if (active)
			gotMsg = PeekMessage(&msg, NULL, 0, 0, TRUE);
		else {
			gotMsg = GetMessage(&msg, NULL, 0, 0);
			if (!gotMsg)
				break;
		}

		if (gotMsg) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			if (msg.message == WM_QUIT)
				break;
		} else if (active) {
//			DWORD time = GetTickCount() - start;
			LARGE_INTEGER counter;
			QueryPerformanceCounter(&counter);
			int time = (int) ((counter.QuadPart - startCounter.QuadPart) * 1000.0f / counterFreq.QuadPart);
			lastTime = time;
			game->update();
			frontEnd->draw(time);
			eglSwapBuffers(display, surface);
			updateMenu(wnd);
		}
		if (game->exit())
			break;
	}

	if (RegCreateKeyEx(ROOT_KEY, KEY_NAME, 0, NULL, REG_OPTION_NON_VOLATILE, 0, NULL, &key, NULL) == ERROR_SUCCESS) {
		if (lastHost)
			RegSetValueEx(key, HOSTNAME_NAME, 0, REG_SZ, (BYTE*) lastHost, sizeof(TCHAR)*(_tcslen(lastHost) + 1));
		if (lastPort)
			RegSetValueEx(key, PORT_NAME, 0, REG_SZ, (BYTE*) lastPort, sizeof(TCHAR)*(_tcslen(lastPort) + 1));
		DWORD val;
		val = frontEnd->getLevel();
		RegSetValueEx(key, LEVEL_NAME, 0, REG_DWORD, (BYTE*) &val, sizeof(val));
		val = frontEnd->getNum();
		RegSetValueEx(key, NUMBER_NAME, 0, REG_DWORD, (BYTE*) &val, sizeof(val));

		RegDeleteKey(key, BLUETOOTH_CACHE_NAME);
		HKEY btCache;
		if (RegCreateKeyEx(key, BLUETOOTH_CACHE_NAME, 0, NULL, REG_OPTION_NON_VOLATILE, 0, NULL, &btCache, NULL) == ERROR_SUCCESS) {
			for (int i = 0; i < bluetoothDevices; i++) {
				TCHAR namebuf[30];
				_stprintf(namebuf, _T("%03d"), i);
				HKEY device;
				if (RegCreateKeyEx(btCache, namebuf, 0, NULL, REG_OPTION_NON_VOLATILE, 0, NULL, &device, NULL) == ERROR_SUCCESS) {
					const TCHAR* name = bluetoothNames[i];
					RegSetValueEx(device, BT_DEVICE_NAME, 0, REG_SZ, (BYTE*) name, sizeof(TCHAR)*(_tcslen(name) + 1));
					BT_ADDR addr = bluetoothAddrs[i];
					BYTE addrBuf[sizeof(BT_ADDR)];
					for (unsigned int j = 0; j < sizeof(BT_ADDR); j++)
						addrBuf[j] = (BYTE) (addr >> (8*(sizeof(BT_ADDR) - 1 - j)));
					RegSetValueEx(device, BT_DEVICE_ADDR, 0, REG_BINARY, addrBuf, sizeof(addrBuf));
					RegCloseKey(device);
				}
			}
			RegCloseKey(btCache);
		}

		RegCloseKey(key);
	}

	


	delete game;
	delete frontEnd;
	delete textFont;
	delete numFont;


	eglMakeCurrent(display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglDestroySurface(display, surface);
	eglDestroyContext(display, context);
	eglTerminate(display);
#ifdef EGL_VERSION_1_2
	eglReleaseThread();
#endif


	return 0;
}
