/*
    Bantumi
    Copyright 2005 - 2007 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#define _WIN32_IE 0x0300
#define WINVER 0x0501
#include <windows.h>
#include <windowsx.h>
#include <tchar.h>

#include <commctrl.h>

#include <stdio.h>
#include <stdlib.h>
#include "resource.h"
#include "tcpip.h"
#include "gui.h"
#include "win32gui.h"
#include "winbt.h"

#ifdef _WIN32_WCE
#include <aygshell.h>
#ifndef SHIDIF_CANCELBUTTON
#define SHIDIF_CANCELBUTTON 0x0080
#endif
#ifndef _MSC_VER
static HMODULE aygdll = NULL;
HRESULT (WINSHELLAPI * SHEnableSoftkey)(HWND hwndMenuBar, UINT uid, BOOL bByIndex, BOOL bEnable);
#endif
extern HWND window;
static bool ppc;
#else
#include <SDL_syswm.h>

#include <ws2bth.h>
#include <BluetoothAPIs.h>
// generally, the desktop ui is most like the ppc ui
#define ppc true
#endif

#define BLUETOOTH_SERVICE_CLASS 0x10273929


char* tocstr(const TCHAR* tstr) {
	int len = _tcslen(tstr);
	char* out = (char*) malloc(len+1);
	char* ptr = out;
	while (*tstr)
		*ptr++ = (char) *tstr++;
	*ptr = '\0';
	return out;
}

TCHAR* fromcstr(const char* str) {
	int len = strlen(str);
	TCHAR* out = (TCHAR*) malloc(sizeof(TCHAR)*(len+1));
	TCHAR* ptr = out;
	while (*str)
		*ptr++ = *str++;
	*ptr = '\0';
	return out;
}


HWND getWindow() {
#ifdef _WIN32_WCE
	return window;
#else
	SDL_SysWMinfo info;
	SDL_VERSION(&info.version);
	if (SDL_GetWMInfo(&info) > 0) {
		return info.window;
	} else {
		return GetForegroundWindow();
	}
#endif
}
#ifndef _WIN32_WCE
static HWND sdlWindow;
static HHOOK hook;
LRESULT CALLBACK windowHook(int nCode, WPARAM wParam, LPARAM lParam) {
	if (nCode == HC_ACTION) {
		CWPSTRUCT *ptr = (CWPSTRUCT*) lParam;
		if (ptr->hwnd == sdlWindow) {
			if (ptr->message == WM_PAINT || ptr->message == WM_ERASEBKGND) {
				redraw();
			}
/*			if (ptr->message == WM_PAINT)
				printf("WM_PAINT\n");
			if (ptr->message == WM_NCPAINT)
				printf("WM_NCPAINT\n");
			if (ptr->message == WM_SYNCPAINT)
				printf("WM_SYNCPAINT\n");
			if (ptr->message == WM_ERASEBKGND)
				printf("WM_ERASEBKGND\n");
*/
		}

/*		if (ptr->message == WM_SETFOCUS) {
			redraw();
		}
*/
	}
	return CallNextHookEx(0, nCode, wParam, lParam);
}

void initRedrawHook() {
	sdlWindow = getWindow();
	hook = SetWindowsHookEx(WH_CALLWNDPROC, windowHook, NULL, GetCurrentThreadId());
}

void removeRedrawHook() {
	UnhookWindowsHookEx(hook);
}
#else
void initRedrawHook() {
}
void removeRedrawHook() {
}
#endif

void flushMessageQueue() {
	MSG msg;
	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

void showWarning(HWND wnd, const char *str) {
	initRedrawHook();
	TCHAR* tstr = fromcstr(str);
	MessageBox(wnd, tstr, _T("Communication error"), MB_OK | MB_ICONWARNING);
	free(tstr);
	removeRedrawHook();
}

void showWarning(const char *str) {
	showWarning(getWindow(), str);
}

static TCHAR buffer1[100], buffer2[100];
const TCHAR *lastPort = NULL;
const TCHAR *lastHost = NULL;
bool client = false;
bool tcpip = true;
#ifdef _WIN32_WCE
static HWND menuBar;
#endif

BOOL CALLBACK TCPDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
#ifdef _WIN32_WCE
	SHMENUBARINFO mbi;
#endif
	switch (uMsg) {
	case WM_INITDIALOG:
#ifdef _WIN32_WCE
		SHINITDLGINFO shidi;
		shidi.dwMask = SHIDIM_FLAGS;
		if (ppc)
			shidi.dwFlags = SHIDIF_CANCELBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN | SHIDIF_EMPTYMENU;
		else
			shidi.dwFlags = SHIDIF_SIZEDLGFULLSCREEN;
		shidi.hDlg = hDlg;
		SHInitDialog(&shidi);

		memset(&mbi, 0, sizeof(mbi));
		mbi.cbSize = sizeof(mbi);
		mbi.hwndParent = hDlg;
		mbi.nToolBarId = client ? CONNECT_MENU : ACCEPT_MENU;
		mbi.hInstRes = GetModuleHandle(NULL);
		mbi.dwFlags = SHCMBF_HMENU;

		if (SHCreateMenuBar(&mbi)) {
			menuBar = mbi.hwndMB;
		}
#endif
		if (lastPort == NULL) {
			_stprintf(buffer1, _T("%d"), DEFAULT_TCP_PORT);
			Edit_SetText(GetDlgItem(hDlg, DIALOG_PORT), buffer1);
		} else
			Edit_SetText(GetDlgItem(hDlg, DIALOG_PORT), lastPort);
		if (client && lastHost)
			Edit_SetText(GetDlgItem(hDlg, DIALOG_HOST), lastHost);
//		ghDialog = hDlg;
		return TRUE;
	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			EndDialog(hDlg, 0);
			return TRUE;
		}
		break;
	case WM_COMMAND:
		if (LOWORD(wParam) == DIALOG_CANCEL || LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg, 0);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_CONNECT_BUTTON) {
			Edit_GetText(GetDlgItem(hDlg, DIALOG_PORT), buffer1, sizeof(buffer1));
			lastPort = buffer1;
			if (client) {
				Edit_GetText(GetDlgItem(hDlg, DIALOG_HOST), buffer2, sizeof(buffer2));
				lastHost = buffer2;
			}
			EndDialog(hDlg, 1);
			return TRUE;
		}
		break;
	case WM_DESTROY:
//		ghDialog = NULL;
		break;
	}
	return FALSE;
}

Connection *showConnectDialog() {
	client = true;
	initRedrawHook();
	int dialog = DIALOG_CONNECT;
	if (!ppc)
		dialog = DIALOG_CONNECT_SP;
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(dialog), getWindow(), TCPDialogProc, 0);
	removeRedrawHook();
	if (retval) {
		const char *err;
		int port;
		// using _stscanf instead of _ttoi, since _wtoi and _wtol aren't in mingw's coredll
		_stscanf(lastPort, _T("%d"), &port);
		char* host = tocstr(lastHost);
		TCPConnection *conn = TCPConnection::startConnect(host, port, &err);
		free(host);
		if (conn == NULL)
			showWarning(err);
		return conn;
	}
	return NULL;
}

Connection *showAcceptDialog() {
	client = false;
	initRedrawHook();
	int dialog = DIALOG_ACCEPT;
	if (!ppc)
		dialog = DIALOG_ACCEPT_SP;
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(dialog), getWindow(), TCPDialogProc, 0);
	removeRedrawHook();
	if (retval) {
		const char *err;
		int port;
		// using _stscanf instead of _ttoi, since _wtoi and _wtol aren't in mingw's coredll
		_stscanf(lastPort, _T("%d"), &port);
		TCPConnection *conn = TCPConnection::startAccept(port, &err);
		if (conn == NULL)
			showWarning(err);
		return conn;
	}
	return NULL;
}

BOOL CALLBACK typeDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
	case WM_INITDIALOG:
		{
#ifdef _WIN32_WCE
		SHINITDLGINFO shidi;
		shidi.dwMask = SHIDIM_FLAGS;
		if (ppc)
			shidi.dwFlags = SHIDIF_CANCELBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN | SHIDIF_EMPTYMENU;
		else
			shidi.dwFlags = SHIDIF_SIZEDLGFULLSCREEN;
		shidi.hDlg = hDlg;
		SHInitDialog(&shidi);

		if (!ppc) {
			SHMENUBARINFO mbi;
			memset(&mbi, 0, sizeof(mbi));
			mbi.cbSize = sizeof(mbi);
			mbi.hwndParent = hDlg;
			if (lParam == DIALOG_TCPIP_BUTTON || lParam == DIALOG_BLUETOOTH_BUTTON)
				mbi.nToolBarId = TYPE_MENU;
			else
				mbi.nToolBarId = CONN_TYPE_MENU;
			mbi.hInstRes = GetModuleHandle(NULL);
			mbi.dwFlags = SHCMBF_HMENU;
			SHCreateMenuBar(&mbi);
			menuBar = mbi.hwndMB;
		}

#endif
		int defctl = 0;
		if (lParam == DIALOG_TCPIP_BUTTON || lParam == DIALOG_BLUETOOTH_BUTTON)
			defctl = client ? DIALOG_JOIN_BUTTON : DIALOG_HOST_BUTTON;
		else
			defctl = tcpip ? DIALOG_TCPIP_BUTTON : DIALOG_BLUETOOTH_BUTTON;
		SendMessage(hDlg, DM_SETDEFID, defctl, 0);
		SendMessage(hDlg, WM_NEXTDLGCTL, (WPARAM)GetDlgItem(hDlg, defctl), TRUE);
		TCHAR title[400];
		GetWindowText(hDlg, title, sizeof(title)/sizeof(title[0]));
		if (lParam == DIALOG_TCPIP_BUTTON)
			_tcscat(title, _T(" - TCP/IP"));
		else if (lParam == DIALOG_BLUETOOTH_BUTTON)
			_tcscat(title, _T(" - Bluetooth"));
		SetWindowText(hDlg, title);
		}
		return FALSE; // to indicate that default keyboard focus shouldn't be set
	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			EndDialog(hDlg, 0);
			return TRUE;
		}
		break;
	case WM_COMMAND:
		if (LOWORD(wParam) == DIALOG_CANCEL || LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg, 0);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_HOST_BUTTON) {
			EndDialog(hDlg, DIALOG_HOST_BUTTON);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_JOIN_BUTTON) {
			EndDialog(hDlg, DIALOG_JOIN_BUTTON);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_TCPIP_BUTTON) {
			EndDialog(hDlg, DIALOG_TCPIP_BUTTON);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_BLUETOOTH_BUTTON) {
			EndDialog(hDlg, DIALOG_BLUETOOTH_BUTTON);
			return TRUE;
		}
		break;
	case WM_DESTROY:
		break;
	}
	return FALSE;
}

#ifdef _WIN32_WCE

int bluetoothDevices = 0;
int maxBluetoothDevices = 0;
BT_ADDR* bluetoothAddrs = NULL;
TCHAR** bluetoothNames = NULL;
int selectedDevice = -1;

void addBluetoothDevice(const TCHAR* name, BT_ADDR addr) {
	if (bluetoothDevices >= maxBluetoothDevices) {
		if (bluetoothAddrs == NULL)
			maxBluetoothDevices = 2;
		else
			maxBluetoothDevices *= 2;

		bluetoothNames = (TCHAR**) realloc(bluetoothNames, sizeof(TCHAR*)*maxBluetoothDevices);
		bluetoothAddrs = (BT_ADDR*) realloc(bluetoothAddrs, sizeof(BT_ADDR)*maxBluetoothDevices);
	}
	bluetoothNames[bluetoothDevices] = _tcsdup(name);
	bluetoothAddrs[bluetoothDevices] = addr;
	bluetoothDevices++;
}

void resetBluetoothDevices() {
	for (int i = 0; i < bluetoothDevices; i++)
		free(bluetoothNames[i]);
	bluetoothDevices = 0;
}

void searchDevices(HWND dialog, HWND list) {
	WSAQUERYSET qs;
	HANDLE query;
	
	SendMessage(list, LB_RESETCONTENT, 0, 0);
	resetBluetoothDevices();

	ZeroMemory(&qs, sizeof(qs));
	qs.dwSize = sizeof(qs);
	qs.dwNameSpace = NS_BTH;
	qs.lpcsaBuffer = NULL;
	
	if (WSALookupServiceBegin(&qs, LUP_CONTAINERS, &query)) {
		showWarning(dialog, getSockError());
		return;
	}
	
	HCURSOR cursor = GetCursor();
	
	while (true) {
		SetCursor(LoadCursor(NULL, IDC_WAIT));

		DWORD size = 2000;
		WSAQUERYSET* res = (WSAQUERYSET*) malloc(size);
		ZeroMemory(res, sizeof(WSAQUERYSET));
		res->dwSize = sizeof(WSAQUERYSET);
		res->dwNameSpace = NS_BTH;

		if (WSALookupServiceNext(query, LUP_RETURN_NAME | LUP_RETURN_ADDR, &size, res)) {
			free(res);
			int err = WSAGetLastError();
			if (err != WSA_E_NO_MORE)
				showWarning(dialog, getSockError(err));
			break;
		}

		if (res->lpszServiceInstanceName) {
			CSADDR_INFO* info = (CSADDR_INFO*) res->lpcsaBuffer;
			SOCKADDR_BTH* sa = (SOCKADDR_BTH*) info->RemoteAddr.lpSockaddr;

			SendMessage(list, LB_ADDSTRING, 0, (LPARAM)res->lpszServiceInstanceName);
			addBluetoothDevice(res->lpszServiceInstanceName, sa->btAddr);
		}

		free(res);

	}
	
	WSALookupServiceEnd(query);
	SetCursor(cursor);
}

BOOL CALLBACK selectDeviceDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	SHMENUBARINFO mbi;
	switch (uMsg) {
	case WM_INITDIALOG:
		{
		SHINITDLGINFO shidi;
		shidi.dwMask = SHIDIM_FLAGS;
		if (ppc)
			shidi.dwFlags = SHIDIF_CANCELBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN | SHIDIF_EMPTYMENU;
		else
			shidi.dwFlags = SHIDIF_SIZEDLGFULLSCREEN;
		shidi.hDlg = hDlg;
		SHInitDialog(&shidi);

//		SendMessage(hDlg, WM_NEXTDLGCTL, (WPARAM)GetDlgItem(hDlg, defctl), TRUE);


		memset(&mbi, 0, sizeof(mbi));
		mbi.cbSize = sizeof(mbi);
		mbi.hwndParent = hDlg;
		mbi.nToolBarId = SELECT_MENU;
		mbi.hInstRes = GetModuleHandle(NULL);
		mbi.dwFlags = SHCMBF_HMENU;
		SHCreateMenuBar(&mbi);
		menuBar = mbi.hwndMB;

		for (int i = 0; i < bluetoothDevices; i++)
			SendMessage(GetDlgItem(hDlg, DIALOG_DEVICE_LIST), LB_ADDSTRING, 0, (LPARAM) bluetoothNames[i]); 

		SHEnableSoftkey(menuBar, DIALOG_CONNECT_BUTTON, FALSE, FALSE);
		if (bluetoothDevices == 0)
			searchDevices(hDlg, GetDlgItem(hDlg, DIALOG_DEVICE_LIST));

		}
		return FALSE; // to indicate that default keyboard focus shouldn't be set
	case WM_SIZE:
		{
		int height = HIWORD(lParam);
		int width = LOWORD(lParam);

		RECT list;
		GetWindowRect(GetDlgItem(hDlg, DIALOG_DEVICE_LIST), &list);
		POINT p;
		p.x = list.left;
		p.y = list.top;
		ScreenToClient(hDlg, &p);
		list.left = p.x;
		list.top = p.y;
		p.x = list.right;
		p.y = list.bottom;
		ScreenToClient(hDlg, &p);
		list.right = p.x;
		list.bottom = p.y;

		int margin = list.left;
		list.bottom = height - margin;
		list.right = width - margin;

		MoveWindow(GetDlgItem(hDlg, DIALOG_DEVICE_LIST), list.left, list.top, list.right - list.left, list.bottom - list.top, TRUE);
		}
		break;
	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			EndDialog(hDlg, 0);
			return TRUE;
		}
		break;
	case WM_COMMAND:
		if ((HWND)lParam == GetDlgItem(hDlg, DIALOG_DEVICE_LIST) && HIWORD(wParam) == LBN_SELCHANGE) {
			int sel = SendMessage(GetDlgItem(hDlg, DIALOG_DEVICE_LIST), LB_GETCURSEL, 0, 0);
			SHEnableSoftkey(menuBar, DIALOG_CONNECT_BUTTON, FALSE, sel != LB_ERR ? TRUE : FALSE);
		}
		if (LOWORD(wParam) == DIALOG_CANCEL || LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg, 0);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_CONNECT_BUTTON) {
			selectedDevice = SendMessage(GetDlgItem(hDlg, DIALOG_DEVICE_LIST), LB_GETCURSEL, 0, 0);
			EndDialog(hDlg, DIALOG_CONNECT_BUTTON);
			return TRUE;
		} else if (LOWORD(wParam) == DIALOG_SEARCH_BUTTON) {
			SHEnableSoftkey(menuBar, DIALOG_CONNECT_BUTTON, FALSE, FALSE);
			searchDevices(hDlg, GetDlgItem(hDlg, DIALOG_DEVICE_LIST));
		}
		break;
	case WM_DESTROY:
		break;
	}
	return FALSE;
}


Connection *showBluetoothConnectDialog() {
	client = true;
	initRedrawHook();
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(DIALOG_SELECT_DEVICE), getWindow(), selectDeviceDialogProc, 0);
	removeRedrawHook();
	if (retval == DIALOG_CONNECT_BUTTON && selectedDevice >= 0 && selectedDevice < bluetoothDevices) {
		const char *err = NULL;
		HCURSOR cursor = GetCursor();
		SetCursor(LoadCursor(NULL, IDC_WAIT));
		Connection* conn = BTConnection::startConnect(bluetoothAddrs[selectedDevice], BLUETOOTH_SERVICE_CLASS, &err);
		SetCursor(cursor);
		if (conn == NULL)
			showWarning(err);
		return conn;
	}

	return NULL;
}

#else
static HMODULE btdll = NULL;
static BOOL (WINAPI *ptr_BluetoothSelectDevices)(BLUETOOTH_SELECT_DEVICE_PARAMS* pbtsdp) = NULL;
static BOOL (WINAPI *ptr_BluetoothSelectDevicesFree)(BLUETOOTH_SELECT_DEVICE_PARAMS* pbtsdp) = NULL;

static HBLUETOOTH_RADIO_FIND (WINAPI *ptr_BluetoothFindFirstRadio)(BLUETOOTH_FIND_RADIO_PARAMS* pbtfrp, HANDLE* phRadio) = NULL;
static BOOL (WINAPI *ptr_BluetoothFindNextRadio)(HBLUETOOTH_RADIO_FIND hFind, HANDLE* phRadio) = NULL;
static BOOL (WINAPI *ptr_BluetoothFindRadioClose)(HBLUETOOTH_RADIO_FIND hFind) = NULL;

Connection *showBluetoothConnectDialog() {
	client = true;
	BLUETOOTH_SELECT_DEVICE_PARAMS btsdp;

	btsdp.dwSize = sizeof(btsdp);
	btsdp.cNumOfClasses = 0;
	btsdp.prgClassOfDevices = NULL;
	btsdp.pszInfo = NULL;
	btsdp.hwndParent = getWindow();
	btsdp.fForceAuthentication = FALSE;
	btsdp.fShowAuthenticated = TRUE;
	btsdp.fShowRemembered = TRUE;
	btsdp.fShowUnknown = TRUE;
	btsdp.fAddNewDeviceWizard = FALSE;
	btsdp.pfnDeviceCallback = NULL;
	btsdp.cNumDevices = 1;
	btsdp.pDevices = NULL;

	initRedrawHook();
	BOOL ok = ptr_BluetoothSelectDevices(&btsdp);
	removeRedrawHook();
	if (ok) {
		if (btsdp.cNumDevices) {
			BLUETOOTH_DEVICE_INFO* btdi = btsdp.pDevices;
			const char *err = NULL;
			Connection* conn = BTConnection::startConnect(btdi->Address.ullLong, BLUETOOTH_SERVICE_CLASS, &err);
			if (conn == NULL)
				showWarning(err);
			ptr_BluetoothSelectDevicesFree(&btsdp);
			return conn;
		}
		ptr_BluetoothSelectDevicesFree(&btsdp);
	}


	return NULL;
}
#endif

Connection *showMultiplayerDialog(bool tcpip) {
	::tcpip = tcpip;
	initRedrawHook();
	int dialog = DIALOG_TYPE;
	if (!ppc)
		dialog = DIALOG_TYPE_SP;
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(dialog), getWindow(), typeDialogProc, tcpip ? DIALOG_TCPIP_BUTTON : DIALOG_BLUETOOTH_BUTTON);
	removeRedrawHook();
	if (retval != 0) {
		if (tcpip) {
			if (retval == DIALOG_HOST_BUTTON)
				return showAcceptDialog();
			else
				return showConnectDialog();
		} else {
			if (retval == DIALOG_HOST_BUTTON) {
				client = false;
				const char *err = NULL;
				Connection* conn = BTConnection::startAccept(BLUETOOTH_SERVICE_CLASS, "Bantumi GL", &err);
				if (conn == NULL)
					showWarning(err);
				return conn;
			} else {
				return showBluetoothConnectDialog();
			}
		}
	}
	return NULL;
}


bool btAvailable() {
#ifdef _WIN32_WCE
	return true;
#else
	if (!BTConnection::available())
		return false;
	if (!ptr_BluetoothSelectDevices || !ptr_BluetoothSelectDevicesFree || !ptr_BluetoothFindFirstRadio || !ptr_BluetoothFindNextRadio || !ptr_BluetoothFindRadioClose)
		return false;

	HANDLE radio;
	BLUETOOTH_FIND_RADIO_PARAMS btfrp;
	btfrp.dwSize = sizeof(btfrp);
	HBLUETOOTH_RADIO_FIND find = ptr_BluetoothFindFirstRadio(&btfrp, &radio);
	if (find) {
		CloseHandle(radio);
		ptr_BluetoothFindRadioClose(find);
		return true;
	}
	return false;
#endif
}

Connection *showMultiplayerDialog() {
	if (!btAvailable())
		return showMultiplayerDialog(true);
	initRedrawHook();
	int dialog = DIALOG_CONN_TYPE;
	if (!ppc)
		dialog = DIALOG_CONN_TYPE_SP;
	int retval = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(dialog), getWindow(), typeDialogProc, 0);
	removeRedrawHook();
	if (retval != 0) {
		if (retval == DIALOG_TCPIP_BUTTON)
			return showMultiplayerDialog(true);
		else
			return showMultiplayerDialog(false);
	}
	return NULL;
}

void initGui() {
#ifdef _WIN32_WCE
#ifndef _MSC_VER
	aygdll = LoadLibrary(_T("aygshell.dll"));
	if (aygdll)
		SHEnableSoftkey = (HRESULT (WINSHELLAPI *)(HWND, UINT, BOOL, BOOL)) GetProcAddress(aygdll, _T("SHEnableSoftkey"));
#endif
	TCHAR platform[20];
	if (SystemParametersInfo(SPI_GETPLATFORMTYPE, sizeof(platform), platform, 0) != 0) {
		if (!_tcscmp(platform, _T("PocketPC")))
			ppc = true;
	}
#else
	INITCOMMONCONTROLSEX controls;
	controls.dwSize = sizeof(controls);
	controls.dwICC = ICC_STANDARD_CLASSES;
	InitCommonControlsEx(&controls);

	btdll = LoadLibrary("irprops.cpl"); // this is never freed...
	if (btdll) {
		ptr_BluetoothSelectDevices = (BOOL (WINAPI *)(BLUETOOTH_SELECT_DEVICE_PARAMS*)) GetProcAddress(btdll, "BluetoothSelectDevices");
		ptr_BluetoothSelectDevicesFree = (BOOL (WINAPI *)(BLUETOOTH_SELECT_DEVICE_PARAMS*)) GetProcAddress(btdll, "BluetoothSelectDevicesFree");
		ptr_BluetoothFindFirstRadio = (HBLUETOOTH_RADIO_FIND (WINAPI *)(BLUETOOTH_FIND_RADIO_PARAMS*,HANDLE*)) GetProcAddress(btdll, "BluetoothFindFirstRadio");
		ptr_BluetoothFindNextRadio = (BOOL (WINAPI *)(HBLUETOOTH_RADIO_FIND,HANDLE*)) GetProcAddress(btdll, "BluetoothFindNextRadio");
		ptr_BluetoothFindRadioClose = (BOOL (WINAPI *)(HBLUETOOTH_RADIO_FIND)) GetProcAddress(btdll, "BluetoothFindRadioClose");
	}
#endif
}

