/*
    Bantumi
    Copyright 2011 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

package st.martin.bantumi;

import java.util.HashSet;
import java.util.Set;

import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;

public class BluetoothSelectDialog {

	public BluetoothSelectDialog(final Activity activity, BluetoothSelectListener listener) {
		mAdapter = BluetoothAdapter.getDefaultAdapter();
		mActivity = activity;
		mListener = listener;
		if (mAdapter == null)
			return;
		final Set<BluetoothDevice> devices = new HashSet<BluetoothDevice>(mAdapter.getBondedDevices());
		final BluetoothDeviceListAdapter adapter = new BluetoothDeviceListAdapter(activity, android.R.layout.select_dialog_item);
		for (BluetoothDevice dev : devices) {
			adapter.add(new ListBluetoothDevice(dev));
		}

		AlertDialog.Builder builder = new AlertDialog.Builder(activity);
		builder.setTitle(R.string.bluetooth_select_title)
		.setAdapter(adapter, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				BluetoothDevice dev = adapter.getItem(which).mDevice;
				mDialog.dismiss();
				mListener.bluetoothDeviceSelected(dev);
			}
		})
		.setCancelable(true)
		.setOnCancelListener(new DialogInterface.OnCancelListener() {
			public void onCancel(DialogInterface dialog) {
				mListener.bluetoothSelectionCancelled();
			}
		});
		
		View buttonView = activity.getLayoutInflater().inflate(R.layout.scan_button, null);
		final Button scan = (Button) buttonView.findViewById(R.id.scan_button);
		scan.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				handleClick();
			}
		});
		builder.setView(buttonView);
		mDialog = builder.create();
		
		mReceiver = new BroadcastReceiver() {
			@Override
			public void onReceive(Context context, Intent intent) {
				String action = intent.getAction();
				if (action.equals(BluetoothAdapter.ACTION_DISCOVERY_STARTED)) {
					mIsScanning = true;
					scan.setText(R.string.cancel_scan_button);
				} else if (action.equals(BluetoothAdapter.ACTION_DISCOVERY_FINISHED)) {
					mIsScanning = false;
					scan.setText(R.string.scan_button);
				} else if (action.equals(BluetoothDevice.ACTION_FOUND)) {
					BluetoothDevice dev = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
					if (!devices.contains(dev)) {
						devices.add(dev);
						adapter.add(new ListBluetoothDevice(dev));
					}
				}
			}
		};
		IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
		filter.addAction(BluetoothAdapter.ACTION_DISCOVERY_STARTED);
		filter.addAction(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
		activity.registerReceiver(mReceiver, filter);
	}

	private Activity mActivity;
	private BluetoothSelectListener mListener;
	private BroadcastReceiver mReceiver;
	private void handleClick() {
		if (!mIsScanning) {
			mAdapter.startDiscovery();
		} else {
			mAdapter.cancelDiscovery();
		}
	}
	
	public void onStop() {
		mActivity.unregisterReceiver(mReceiver);
	}
	
	private boolean mIsScanning;
	private BluetoothAdapter mAdapter;
	private AlertDialog mDialog;
	public AlertDialog getDialog() {
		return mDialog;
	}

	public class ListBluetoothDevice {
		public ListBluetoothDevice(BluetoothDevice dev) {
			mDevice = dev;
		}
		private BluetoothDevice mDevice;
		@Override
		public String toString() {
			String name = mDevice.getName();
			if (name != null && name.length() > 0)
				return name;
			return mDevice.toString();
		}
	}
	private static final class BluetoothDeviceListAdapter extends ArrayAdapter<ListBluetoothDevice> {
		public BluetoothDeviceListAdapter(Context context, int textViewResourceId) {
			super(context, textViewResourceId);
		}
	}
	
}
