/*
    Bantumi
    Copyright 2011 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

package st.martin.bantumi;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.concurrent.LinkedBlockingQueue;

import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.Context;

class ConnectionThread extends Thread {
	private final ConnectionManager mManager;
	private final Context mContext;
	public ConnectionThread(ConnectionManager manager, Context context, BluetoothServerSocket s) {
		mManager = manager;
		mContext = context;
		mServerSocket = s;
		start();
	}
	public ConnectionThread(ConnectionManager manager, Context context, BluetoothSocket s) {
		mManager = manager;
		mContext = context;
		mSocket = s;
		start();
	}
	private BluetoothServerSocket mServerSocket;
	private BluetoothSocket mSocket;
	private InputStream mInput;
	private OutputStream mOutput;
	private Thread mOutputThread;
	public void run() {
		try {
			boolean client = mServerSocket == null;
			mManager.initJavaConnection(client);
			if (mServerSocket != null) {
				BluetoothSocket s = mServerSocket.accept();
				synchronized (this) {
					mSocket = s;
					mServerSocket.close();
					mServerSocket = null;
				}
			} else {
				mSocket.connect();
			}
			mManager.setConnectionStatus(true, null);
			mInput = mSocket.getInputStream();
			mOutput = mSocket.getOutputStream();
			mOutputThread = new Thread() {
				public void run() {
					while (true) {
						byte[] buffer;
						try {
							buffer = mSendQueue.take();
						} catch (InterruptedException ie) {
							ie.printStackTrace();
							continue;
						}
						if (mClosing)
							break;
						try {
							mOutput.write(buffer);
						} catch (IOException ioe) {
							mManager.setConnectionStatus(false, ioe.getMessage());
							break;
						}
					}
				}
			};
			mOutputThread.start();
			byte[] buffer = new byte[100];
			while (true) {
				int n = mInput.read(buffer);
				if (n <= 0)
					break;
				mManager.handleInputData(buffer, n);
			}
			mManager.setConnectionStatus(false, mContext.getResources().getString(R.string.connection_closed_msg));
		} catch (IOException ioe) {
			mManager.setConnectionStatus(false, ioe.getMessage());
		}
	}
	private LinkedBlockingQueue<byte[]> mSendQueue = new LinkedBlockingQueue<byte[]>();
	private boolean mClosing = false;
	public void writeOutputData(byte[] buffer) {
		mSendQueue.add(buffer);
	}
	public void close() {
		mClosing = true;
		mSendQueue.add(new byte[1]);
		synchronized (this) {
			try {
				if (mServerSocket != null)
					mServerSocket.close();
				if (mSocket != null)
					mSocket.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}
	}
}